/*
 *  UltimateIRCd - an Internet Relay Chat Daemon, src/dynconf.c
 *
 *  Copyright (C) 2002 by the past and present ircd coders, and others.
 *  Refer to the documentation within doc/authors/ for full credits and copyrights.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *  USA
 *
 *  $Id: dynconf.c,v 1.10 2002/11/18 00:40:07 shadowmaster Exp $
 *
 */

#define DYNCONF_C
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "userload.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifndef _WIN32
#include <utmp.h>
#else
#include <io.h>
#endif
#include <fcntl.h>
#include "h.h"

#define DoDebug fprintf(stderr, "[%s] %s | %li\n", babuf, __FILE__, __LINE__);
#define AllocCpy(x,y) if ((x) && type == 1) MyFree((x)); x = (char *) MyMalloc(strlen(y) + 1); strcpy(x,y)
#define XtndCpy(x,y) x = (char *) MyMalloc(strlen(y) + 2); *x = '\0'; strcat(x, "*"); strcpy(x,y)

/* internals */
aConfiguration iConf;
int icx = 0;
char buf[1024];


/* strips \r and \n's from the line */
void
iCstrip (char *line)
{
  char *c;

  if ((c = strchr (line, '\n')))
    *c = '\0';
  if ((c = strchr (line, '\r')))
    *c = '\0';
}

/* this loads dynamic DCONF */
int
load_conf (char *filename, int type)
{
  FILE *zConf;
  char *version = NULL;
  char *i;

  zConf = fopen (filename, "r");

  if (!zConf)
    {
      if (type == 1)
	{
	  sendto_ops ("DynConf Fatal Error: Could not load %s !", filename);
	  return -1;
	}
      else
	{
	  fprintf (stderr, "              ~~ ERROR ~~\n");
	  fprintf (stderr, " DynConf Fatal Error: Couldn't load %s !!!\n",
		   filename);
	  fprintf (stderr,
		   "~~~~~~~~~~~~~~~~~~~~~~~~\n");

	  exit (-1);
	}
    }
  i = fgets (buf, 1023, zConf);
  if (!i)
    {
      if (type == 1)
	{
	  sendto_ops ("DynConf Fatal Error: Error reading from %s !",
		      filename);
	  return -1;
	}
      else
	{
	  fprintf (stderr, "              ~~ ERROR ~~\n");
	  fprintf (stderr, " DynConf Fatal Error: Couldn't read %s !!!\n",
		   filename);
	  fprintf (stderr,
		   "~~~~~~~~~~~~~~~~~~~~~~~~\n");
	  exit (-1);
	}
    }
  iCstrip (buf);
  version = strtok (buf, "^");
  version = strtok (NULL, "");
  /* is this an ircd.ini file? */
  if (!match ("1.*", version))
    {
      /* wrong version */
      if (strcmp (version, DYNCONF_CONF_VERSION))
	{
	  if (type == 1)
	    {
	      sendto_ops
		("DynConf Fatal Error: %s got a non-compatible version (%s) !",
		 filename, version);
	      return -1;
	    }
	  else
	    {
	      fprintf (stderr, "              ~~ ERROR ~~\n");
	      fprintf (stderr,
		       " DynConf Fatal Error: Incompatible version (%s) in %s - please upgrade !!!\n",
		       version, filename);
	      fprintf (stderr,
		       "~~~~~~~~~~~~~~~~~~~~~~~~\n");
	      exit (-1);
	    }
	}
      load_conf2 (zConf, filename, type);
      return -1;
    }
  else if (!match ("2.*", version))
    {
      /* network file */
      /* wrong version */
      if (strcmp (version, DYNCONF_NETWORK_VERSION))
	{
	  if (type == 1)
	    {
	      sendto_ops
		("DynConf Fatal Error: %s got a non-compatible network file version (%s) !",
		 filename, version);
	      return -1;
	    }
	  else
	    {
	      fprintf (stderr, "              ~~ ERROR ~~\n");
	      fprintf (stderr,
		       " DynConf Fatal Error: Incompatible network file version (%s) in %s - please upgrade !!!\n",
		       version, filename);
	      fprintf (stderr,
		       "~~~~~~~~~~~~~~~~~~~~~~~~\n");
	      exit (-1);
	    }
	}
      load_conf3 (zConf, filename, type);
      return -1;
    }
  else
    {
      if (type == 1)
	{
	  sendto_ops ("DynConf Fatal Error: Malformed version header in %s.",
		      filename);
	  return -1;
	}
      else
	{
	  fprintf (stderr, "              ~~ ERROR ~~\n");
	  fprintf (stderr,
		   " DynConf Fatal Error: Malformed or missing version header in %s.\n",
		   filename);
	  fprintf (stderr,
		   "~~~~~~~~~~~~~~~~~~~~~~~~\n");
	  exit (-1);
	}
      return -1;
    }
  return -1;
}

int
load_conf2 (FILE * conf, char *filename, int type)
{
  char *databuf = buf;
  char *stat = databuf;
  char *p, *setto, *var;

  if (!conf)
    return -1;

  *databuf = '\0';

  /* loop to read data */
  while (stat != NULL)
    {
      stat = fgets (buf, 1020, conf);
      if ((*buf == '#') || (*buf == '/'))
	continue;

      iCstrip (buf);
      if (*buf == '\0')
	continue;
      p = strtok (buf, " ");
      /*
       ** We want this to be case insensitive so we use stricmp instead of strcmp - HAMLET
       */
      if (stricmp (p, "Include") == 0)
	{
	  strtok (NULL, " ");
	  setto = strtok (NULL, "");
	  /* We need this for STATS S -codemastr 
	     Isn't there a better way to show all include files? --Stskeeps */
	  AllocCpy (INCLUDE, setto);
	  load_conf (setto, type);
	}
      /*
       ** We want this to be case insensitive so we use stricmp instead of strcmp - HAMLET
       */
      else if (stricmp (p, "Set") == 0)
	{
	  var = strtok (NULL, " ");
	  if (var == NULL)
	    continue;
	  if (*var == '\0')
	    continue;

	  strtok (NULL, " ");
	  setto = strtok (NULL, "");
	  /*
	   * HALFHUB
	   */

	  if (strcmp (var, "HALFHUB") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "0";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set halfhub in ircd.ini is invalid (NULL value). Resetting it to default.\n");
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set halfhub in ircd.ini is invalid (NULL value). Resetting to default.");
		}
	      if (strchr (setto, ' '))
		{
		  setto = "0";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set halfhub in ircd.ini is invalid (Contains Spaces). Resetting it to default.\n");
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set halfhub in ircd.ini is invalid (Contains Spaces). Resetting it to default.");
		}
	      HALFHUB = atoi (setto);
	    }
	  /*
	   * TRUEHUB
	   */
	  else if (strcmp (var, "TRUEHUB") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "0";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set truehub in ircd.ini is invalid (NULL value). Resetting it to default.\n");
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set truehub in ircd.ini is invalid (NULL value). Resetting to default.");
		}
	      if (strchr (setto, ' '))
		{
		  setto = "0";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set truehub in ircd.ini is invalid (Contains Spaces). Resetting it to default.\n");
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set truehub in ircd.ini is invalid (Contains Spaces). Resetting it to default.");
		}
	      TRUEHUB = atoi (setto);
	    }
	  /*
	   * CLIENT_FLOOD
	   */
	  else if (strcmp (var, "CLIENT_FLOOD") == 0)
	    {
	      CLIENT_FLOOD = atol (setto);
	      if (CLIENT_FLOOD > 8000)
		{
		  fprintf (stderr,
			   " DynConf Warning: CLIENT_FLOOD set to large. Resetting it to 8000\n");
		  CLIENT_FLOOD = atol ("8000");
		}
	      if (CLIENT_FLOOD < 512)
		{
		  fprintf (stderr,
			   " DynConf Warning: CLIENT_FLOOD set to low. Resetting it to 4000\n");
		  CLIENT_FLOOD = atol ("4000");
		}
	    }
	  /*
	   * GEO_LOCATION
	   */
	  else if (strcmp (var, "geo_location") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "0";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set geo_location in ircd.ini is invalid (NULL value). Resetting it to default.\n");
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set geo_location in ircd.ini is invalid (NULL value). Resetting to default.");
		}
	      AllocCpy (geo_location, setto);
	    }
	  /*
	   * DEF_MODE_I
	   */
	  else if (strcmp (var, "DEF_MODE_I") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "0";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set def_mode_i in ircd.ini is invalid (NULL value). Resetting it to default.\n");
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set def_mode_i in ircd.ini is invalid (NULL value). Resetting to default.");
		}
	      if (strchr (setto, ' '))
		{
		  setto = "0";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set def_mode_i in ircd.ini is invalid (Contains Spaces). Resetting it to default.\n");
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set def_mode_i in ircd.ini is invalid (Contains Spaces). Resetting it to default.");
		}
	      DEF_MODE_I = atoi (setto);
	    }
	  /*
	   * DEF_MODE_S
	   */
	  else if (strcmp (var, "DEF_MODE_S") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "0";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set def_mode_s in ircd.ini is invalid (NULL value). Resetting it to default.\n");
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set def_mode_s in ircd.ini is invalid (NULL value). Resetting to default.");
		}
	      if (strchr (setto, ' '))
		{
		  setto = "0";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set def_mode_s in ircd.ini is invalid (Contains Spaces). Resetting it to default.\n");
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set def_mode_s in ircd.ini is invalid (Contains Spaces). Resetting it to default.");
		}
	      DEF_MODE_S = atoi (setto);
	    }
	  /*
	   * DEF_MODE_W
	   */
	  else if (strcmp (var, "DEF_MODE_W") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "0";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set def_mode_w in ircd.ini is invalid (NULL value). Resetting it to default.\n");
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set def_mode_w in ircd.ini is invalid (NULL value). Resetting to default.");
		}
	      if (strchr (setto, ' '))
		{
		  setto = "0";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set def_mode_w in ircd.ini is invalid (Contains Spaces). Resetting it to default.\n");
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set def_mode_w in ircd.ini is invalid (Contains Spaces). Resetting it to default.");
		}
	      DEF_MODE_W = atoi (setto);
	    }
	  /*
	   * SHORT_MOTD
	   */
	  else if (strcmp (var, "SHORT_MOTD") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "0";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set short_motd in ircd.ini is invalid (NULL value). Resetting it to default.\n");
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set short_motd in ircd.ini is invalid (NULL value). Resetting to default.");
		}
	      if (strchr (setto, ' '))
		{
		  setto = "0";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set short_motd in ircd.ini is invalid (Contains Spaces). Resetting it to default.\n");
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set short_motd in ircd.ini is invalid (Contains Spaces). Resetting it to default.");
		}
	      SHORT_MOTD = atoi (setto);
	    }
	  else if (strcmp (var, "CONFIG_FILE_STOP") == 0)
	    {
	      if (atoi (setto) == 1)
		{
		  fprintf (stderr, "              ~~ ERROR ~~\n");
		  fprintf (stderr,
			   " DynConf Fatal Error: File stop code recieved in %s!!!\n",
			   filename);
		  fprintf (stderr,
			   " DynConf Fatal Error: You MUST edit your %s file!!!\n",
			   filename);
		  fprintf (stderr,
			   "~~~~~~~~~~~~~~~~~~~~~~~~\n");
		  exit (1);
		}
	    }
	  else
	    {
	      if (type == 0)
		fprintf (stderr,
			 " Dynconf Warning: Invalid Set %s %s setting in %s \n",
			 var, setto, filename);
	      else
		sendto_realops
		  ("Dynconf Warning: Invalid Set %s %s setting in %s \n", var,
		   setto, filename);
	    }

	}
      else
	{

	  if (type == 0)
	    {
	      fprintf (stderr,
		       " Dynconf Warning: %s in %s is not a known setting to me!\n",
		       p, filename);
	    }
	  else
	    {
	      sendto_realops
		("Dynconf Warning: %s in %s is not a known setting to me!\n",
		 p, filename);
	    }
	}
    }
  if (type == 0)
    {
      fprintf (stderr, " Dynconf Loaded %s\n", filename);
    }
  else
    {
      sendto_realops ("Loaded %s", filename);
    }
}

/* Load .network options */
int
load_conf3 (FILE * conf, char *filename, int type)
{
  char *databuf = buf;
  char *stat = databuf;
  char *p, *setto, *var;
  if (!conf)
    return -1;

  *databuf = '\0';

  /* loop to read data */
  while (stat != NULL)
    {
      stat = fgets (buf, 1020, conf);
      if ((*buf == '#') || (*buf == '/'))
	continue;

      iCstrip (buf);
      if (*buf == '\0')
	continue;

      p = strtok (buf, " ");
      /*
       ** We want this to be case insensitive so we use stricmp instead of strcmp - HAMLET
       */
      if (stricmp (p, "Set") == 0)
	{
	  var = strtok (NULL, " ");
	  if (var == NULL)
	    continue;
	  if (*var == '\0')
	    continue;

	  strtok (NULL, " ");
	  setto = strtok (NULL, "");
	  /*
	   * SHORT_MOTD
	   */
	  if (strcmp (var, "ircnetwork") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "SomeIRC";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set ircnetwork in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set ircnetwork in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "SomeIRC";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set ircnetwork in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set ircnetwork in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      AllocCpy (ircnetwork, setto);
	    }
	  /*
	   * IRCNETWORK_NAME
	   */
	  else if (strcmp (var, "ircnetwork_name") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "The Some IRC Network";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set ircnetwork_name in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set ircnetwork_name in %s is invalid (NULL value). Resetting to default.");
		}
	      AllocCpy (ircnetwork_name, setto);
	    }
	  /*
	   * DEFSERV
	   */
	  else if (strcmp (var, "defserv") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "irc.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set defserv in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set defserv in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "irc.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set defserv in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set defserv in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      AllocCpy (defserv, setto);
	    }
	  /*
	   * WEBPAGE
	   */
	  else if (strcmp (var, "webpage") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "http://www.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set webpage in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set webpage in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "http://www.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set webpage in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set webpage in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      AllocCpy (webpage, setto);
	    }
	  /*
	   * WEBPAGE_RULES
	   */
	  else if (strcmp (var, "webpage_rules") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "http://www.someirc.net/rules/";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set webpage_rules in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set webpage_rules in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "http://www.someirc.net/rules/";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set webpage_rules in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set webpage_rules in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      AllocCpy (webpage_rules, setto);
	    }
	  /*
	   * KLINE_ADDRESS
	   */
	  else if (strcmp (var, "KLINE_ADDRESS") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "kline@someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set kline_address in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set kline_address in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "kline@someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set kline_address in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set kline_address in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      if (!strchr (setto, '@'))
		{
		  setto = "kline@someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set kline_address in %s is invalid (Is not a email). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set kline_address in %s is invalid (Is not a email). Resetting it to default.",
		       INCLUDE);
		}
	      AllocCpy (KLINE_ADDRESS, setto);
	    }
	  /*
	   * MODE_X
	   */
	  else if (strcmp (var, "MODE_x") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "1";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set mode_x in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set mode_x in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "1";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set mode_x in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set mode_x in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      MODE_x = atoi (setto);
	    }
	  /*
	   * NET_CLONE_PROTECTION
	   */
	  else if (strcmp (var, "net_clone_protection") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "1";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set net_clone_protection in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set net_clone_protection in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "1";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set net_clone_protection in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set net_clone_protection in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      net_clone_protection = atoi (setto);
	    }
	  /*
	   * NET_CLONE_LIMIT
	   */
	  else if (strcmp (var, "net_clone_limit") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "5";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set net_clone_limit in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set net_clone_limit in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "5";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set net_clone_limit in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set net_clone_limit in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      net_clone_limit = atol (setto);
	    }
	  /*
	   * NET_CLONE_LIMIT_BYPASS
	   */
	  else if (strcmp (var, "net_clone_limit_bypass") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "1";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set net_clone_limit_bypass in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set net_clone_limit_bypass in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "1";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set net_clone_limit_bypass in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set net_clone_limit_bypass in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      net_clone_limit_bypass = atoi (setto);
	    }
	  /*
	   * HIDE_ULINES
	   */
	  else if (strcmp (var, "HIDE_ULINES") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "1";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set hide_ulines in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set hide_ulines in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "1";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set hide_ulines in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set hide_ulines in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      HIDE_ULINES = atoi (setto);
	    }
	  /*
	   * GLOBAL_CONNECTS
	   */
	  else if (strcmp (var, "global_connects") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "1";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set global_connects in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set global_connects in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "1";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set global_connects in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set global_connects in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      GLOBAL_CONNECTS = atoi (setto);
	    }
	  /*
	   * NETWORK_PROXYSCAN_NOTICE
	   */
	  else if (strcmp (var, "network_proxyscan_notice") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "0";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set network_proxyscan_notice in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set network_proxyscan_notice in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "0";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set network_proxyscan_notice in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set network_proxyscan_notice in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      network_proxyscan_notice = atoi (setto);
	    }
	  /*
	   * NETWORK_AUTO_PROTECT
	   */
	  else if (strcmp (var, "network_auto_protect") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "0";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set network_auto_protect in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set network_auto_protect in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "0";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set network_auto_protect in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set network_auto_protect in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      network_auto_protect = atoi (setto);
	    }
	  /*
	   * NET_USERS_AUTO_JOIN
	   */
	  else if (strcmp (var, "net_users_auto_join") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "0";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set net_users_auto_join in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set net_users_auto_join in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "0";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set net_users_auto_join in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set network_users_auto_join in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      AllocCpy (net_users_auto_join, setto);
	    }
	  /*
	   * NET_OPERS_AUTO_JOIN
	   */
	  else if (strcmp (var, "net_opers_auto_join") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "0";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set net_opers_auto_join in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set net_opers_auto_join in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "0";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set net_opers_auto_join in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set network_opers_auto_join in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      AllocCpy (net_opers_auto_join, setto);
	    }
	  /*
	   * SERVICES_NAME
	   */
	  else if (strcmp (var, "services_name") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "services.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set services_name in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set services_name in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "services.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set services_name in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set services_name in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      AllocCpy (services_name, setto);
	    }
	  /*
	   * WATCHDOG_NAME
	   */
	  else if (strcmp (var, "watchdog_name") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "watchdog.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set watchdog_name in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set watchdog_name in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "watchdog.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set watchdog_name in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set watchdog_name in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      AllocCpy (watchdog_name, setto);
	    }
	  /*
	   * STATSERV_NAME
	   */
	  else if (strcmp (var, "statserv_name") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "statistics.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set statserv_name in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set statserv_name in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "statistics.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set statserv_name in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set statserv_name in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      AllocCpy (statserv_name, setto);
	    }
	  /*
	   * LOCOP_HOST
	   */
	  else if (strcmp (var, "locop_host") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "LocalOper.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set locop_host in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set locop_host in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "LocalOper.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set locop_host in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set locop_host in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      AllocCpy (locop_host, setto);
	    }
	  /*
	   * OPER_HOST
	   */
	  else if (strcmp (var, "oper_host") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "Oper.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set oper_host in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set oper_host in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "Oper.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set oper_host in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set oper_host in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      AllocCpy (oper_host, setto);
	    }
	  /*
	   * ADMIN_HOST
	   */
	  else if (strcmp (var, "admin_host") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "Admin.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set admin_host in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set admin_host in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "Admin.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set admin_host in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set admin_host in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      AllocCpy (admin_host, setto);
	    }
	  /*
	   * SERVADMIN_HOST
	   */
	  else if (strcmp (var, "servadmin_host") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "ServerAdmin.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set servadmin_host in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set servadmin_host in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "ServerAdmin.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set servadmin_host in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set servadmin_host in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      AllocCpy (servadmin_host, setto);
	    }
	  /*
	   * SERVCESOPER_HOST
	   */
	  else if (strcmp (var, "servicesoper_host") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "ServicesOper.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set servicesoper_host in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set servicesoper_host in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "ServicesOper.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set servicesoper_host in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set servicesoper_host in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      AllocCpy (servicesoper_host, setto);
	    }
	  /*
	   * SERVICESADMIN_HOST
	   */
	  else if (strcmp (var, "servicesadmin_host") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "ServicesAdmin.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set servicesadmin_host in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set servicesadmin_host in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "ServicesAdmin.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set servicesadmin_host in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set servicesadmin_host in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      AllocCpy (servicesadmin_host, setto);
	    }
	  /*
	   * SERVICESROOT_HOST
	   */
	  else if (strcmp (var, "servicesroot_host") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "ServicesRoot.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set servicesroot_host in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set servicesroot_host in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "ServicesRoot.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set servicesroot_host in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set servicesroot_host in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      AllocCpy (servicesroot_host, setto);
	    }
	  /*
	   * TECHADMIN_HOST
	   */
	  else if (strcmp (var, "techadmin_host") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "TechAdmin.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set techadmin_host in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set techadmin_host in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "TechAdmin.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set techadmin_host in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set techadmin_host in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      AllocCpy (techadmin_host, setto);
	    }
	  /*
	   * NETADMIN_HOST
	   */
	  else if (strcmp (var, "netadmin_host") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "NetAdmin.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set netadmin_host in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set netadmin_host in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "NetAdmin.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set netadmin_host in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set netadmin_host in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      AllocCpy (netadmin_host, setto);
	    }
	  /*
	   * IRCADMIN_HOST
	   */
	  else if (strcmp (var, "ircadmin_host") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "IRCAdmin.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set ircadmin_host in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set ircadmin_host in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "IRCAdmin.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set ircadmin_host in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set ircadmin_host in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      AllocCpy (ircadmin_host, setto);
	    }
	  /*
	   * SBOT_HOST
	   */
	  else if (strcmp (var, "sbot_host") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "serverbot.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set sbot_host in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set sbot_host in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "serverbot.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set sbot_host in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set sbot_host in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      AllocCpy (sbot_host, setto);
	    }
	  /*
	   * NETMON_HOST
	   */
	  else if (strcmp (var, "netmon_host") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "network-monitor.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set netmon_host in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set netmon_host in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "network-monitor.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set netmon_host in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set netmon_host in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      AllocCpy (netmon_host, setto);
	    }
	  /*
	   * HELPCHAN
	   */
	  else if (strcmp (var, "helpchan") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "#help";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set helpchan in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set helpchan in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "#help";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set helpchan in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set helpchan in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      if (!strchr (setto, '#'))
		{
		  setto = "#help";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set helpchan in %s is invalid (Is not a Channel). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set helpchan in %s is invalid (Is not a Channel). Resetting it to default.",
		       INCLUDE);
		}
	      AllocCpy (helpchan, setto);
	    }
	  /*
	   * NET_ENFORCER
	   */
	  else if (strcmp (var, "net_enforcer") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "enforcer";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set net_enforcer in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set net_enforcer in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "enforcer";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set net_enforcer in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set net_enforcer in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      AllocCpy (net_enforcer, setto);
	    }
	  /*
	   * NET_ENFORCER_USERID
	   */
	  else if (strcmp (var, "net_enforcer_userid") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "enforcer";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set net_enforcer_userid in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set net_enforcer_userid in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "enforcer";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set net_enforcer_userid in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set net_enforcer_userid in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      AllocCpy (net_enforcer_userid, setto);
	    }
	  /*
	   * NET_ENFORCER_HOST
	   */
	  else if (strcmp (var, "net_enforcer_host") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "enforcer.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set net_enforcer_host in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set net_enforcer_host in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "enforcer.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set net_enforcer_host in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set net_enforcer_host in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      AllocCpy (net_enforcer_host, setto);
	    }
	  /*
	   * NET_ENFORCER_REALNAME
	   */
	  else if (strcmp (var, "net_enforcer_realname") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "enforcer";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set net_enforcer_realname in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set net_enforcer_realname in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      AllocCpy (net_enforcer_realname, setto);
	    }
	  /*
	   * NET_ENFORCER_SERVER
	   */
	  else if (strcmp (var, "net_enforcer_server") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "enforcer.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set net_enforcer_server in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set net_enforcer_server in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      if (strchr (setto, ' '))
		{
		  setto = "enforcer.someirc.net";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set net_enforcer_server in %s is invalid (Contains Spaces). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set net_enforcer_server in %s is invalid (Contains Spaces). Resetting it to default.",
		       INCLUDE);
		}
	      AllocCpy (net_enforcer_server, setto);
	    }
	  /*
	   * NET_ENFORCER_SERVER_NAME
	   */
	  else if (strcmp (var, "net_enforcer_server_name") == 0)
	    {
	      if (setto == NULL)
		{
		  setto = "The Somenet Enforcer";

		  if (type == 0)
		    fprintf (stderr,
			     " Dynconf ERROR: Set net_enforcer_server_name in %s is invalid (NULL value). Resetting it to default.\n",
			     INCLUDE);
		  else
		    sendto_realops
		      ("Dynconf ERROR: Set net_enforcer_server_name in %s is invalid (NULL value). Resetting to default.",
		       INCLUDE);
		}
	      AllocCpy (net_enforcer_server_name, setto);
	    }
	  else
	    {
	      if (type == 0)
		fprintf (stderr,
			 " Dynconf Warning: Invalid Set %s %s setting in %s \n",
			 var, setto, INCLUDE);
	      else
		sendto_realops
		  ("Dynconf Warning: Invalid Set %s %s setting in %s \n", var,
		   setto, INCLUDE);
	    }

	}
      else
	{

	  if (type == 0)
	    {
	      fprintf (stderr,
		       " Dynconf Warning: %s in %s is not a known setting to me!\n",
		       p, INCLUDE);
	    }
	  else
	    {
	      sendto_realops
		("Dynconf Warning: %s in %s is not a known setting to me!\n",
		 p, INCLUDE);
	    }
	}

    }
  if (type == 0)
    {
      fprintf (stderr, " Dynconf Loaded %s\n", INCLUDE);
    }
  else
    {
      sendto_realops ("Loaded %s", INCLUDE);
    }
}


void
doneconf (int type)
{
  /* calculate on errors */
  char *errormsg = MyMalloc (16384);

  *errormsg = '\0';
  if (!KLINE_ADDRESS || (*KLINE_ADDRESS == '\0'))
    strcat (errormsg, "- Missing KLINE_ADDRESS\n");
  if (KLINE_ADDRESS)
    if (!strchr (KLINE_ADDRESS, '@') && !strchr (KLINE_ADDRESS, ':'))
      {
	strcat (errormsg, "- KLINE_ADDRESS is not an e-mail or an URL\n");
      }
#define Missing(x) !(x) || (*(x) == '\0')
  if (Missing (defserv))
    strcat (errormsg, "- Missing defserv field\n");
  if (Missing (services_name))
    strcat (errormsg, "- Missing services_name field\n");
  if (Missing (oper_host))
    strcat (errormsg, "- Missing oper_host field\n");
  if (Missing (admin_host))
    strcat (errormsg, "- Missing admin_host field\n");
  if (Missing (locop_host))
    strcat (errormsg, "- Missing locop_host field\n");
  if (Missing (netadmin_host))
    strcat (errormsg, "- Missing netadmin_host field\n");
  if (Missing (techadmin_host))
    strcat (errormsg, "- Missing techadmin_host field\n");
  if (Missing (helpchan))
    strcat (errormsg, "- Missing helpchan field\n");
  if (Missing (statserv_name))
    strcat (errormsg, "- Missing statserv_name field\n");
  if (*errormsg != '\0')
    {
      fprintf (stderr,
	       "\n*** ERRORS IN DYNAMIC CONFIGURATION (listed below) ***\n");
      fprintf (stderr, errormsg);
      MyFree (errormsg);
      fprintf (stderr,
	       "~~~~~~~~~~~~~~~~~~~~~~~~\n");
      exit (-1);
    }
  MyFree (errormsg);
}

void
init_dynconf (void)
{
  bzero (&iConf, sizeof (iConf));
}

/* Report the unrealircd.conf info -codemastr*/
void
report_dynconf (aClient * sptr)
{
  sendto_one (sptr,
	      ":%s %d %s :\2~~~~~~~~ Dynconf Settings ~~~~~~~~\2",
	      me.name, RPL_SETTINGS, sptr->name);

  sendto_one (sptr, ":%s %d %s :", me.name, RPL_SETTINGS, sptr->name);


  sendto_one (sptr, ":%s %d %s :Include: \2%s\2",
	      me.name, RPL_SETTINGS, sptr->name, INCLUDE);

  sendto_one (sptr, ":%s %d %s :Default Port: \2%d\2",
	      me.name, RPL_SETTINGS, sptr->name, portnum);

  sendto_one (sptr, ":%s %d %s :Half HUB: \2%i\2",
	      me.name, RPL_SETTINGS, sptr->name, HALFHUB);

  sendto_one (sptr, ":%s %d %s :True HUB: \2%i\2",
	      me.name, RPL_SETTINGS, sptr->name, TRUEHUB);

  sendto_one (sptr, ":%s %d %s :Flood Level: \2%d\2",
	      me.name, RPL_SETTINGS, sptr->name, CLIENT_FLOOD);

  sendto_one (sptr, ":%s %d %s :Geo Location: \2%s\2",
	      me.name, RPL_SETTINGS, sptr->name, geo_location);

  sendto_one (sptr, ":%s %d %s :Usermode i on login: \2%i\2",
	      me.name, RPL_SETTINGS, sptr->name, DEF_MODE_I);

  sendto_one (sptr, ":%s %d %s :Usermode s on login: \2%i\2",
	      me.name, RPL_SETTINGS, sptr->name, DEF_MODE_S);

  sendto_one (sptr, ":%s %d %s :Usermode w on login: \2%i\2",
	      me.name, RPL_SETTINGS, sptr->name, DEF_MODE_W);

  sendto_one (sptr, ":%s %d %s :Short MOTD: \2%i\2",
	      me.name, RPL_SETTINGS, sptr->name, SHORT_MOTD);

  sendto_one (sptr, ":%s %d %s : ", me.name, RPL_SETTINGS, sptr->name);

  sendto_one (sptr,
	      ":%s %d %s :\2~~~~~~~~~~~~~~~~~~~~~~~~~~\2",
	      me.name, RPL_SETTINGS, sptr->name);

  sendto_one (sptr, rpl_str (RPL_ENDOFSETTINGS), me.name, sptr->name);
}

/* Report the network file info -codemastr */
void
report_network (aClient * sptr)
{
  sendto_one (sptr,
	      ":%s %d %s :\2~~~~~~~~ Network Info ~~~~~~~~\2",
	      me.name, RPL_SETTINGS, sptr->name);

  sendto_one (sptr, ":%s %d %s : ", me.name, RPL_SETTINGS, sptr->name);

  sendto_one (sptr, ":%s %d %s :Network File: \2%s\2",
	      me.name, RPL_SETTINGS, sptr->name, ircnetwork);

  sendto_one (sptr, ":%s %d %s :Network Name: \2%s\2",
	      me.name, RPL_SETTINGS, sptr->name, ircnetwork_name);

  sendto_one (sptr, ":%s %d %s :Default Server: \2%s\2",
	      me.name, RPL_SETTINGS, sptr->name, defserv);

  sendto_one (sptr, ":%s %d %s :Network Web Site: \2%s\2",
	      me.name, RPL_SETTINGS, sptr->name, webpage);

  sendto_one (sptr, ":%s %d %s :Network Rules Page: \2%s\2",
	      me.name, RPL_SETTINGS, sptr->name, webpage_rules);

  sendto_one (sptr, ":%s %d %s :K-Line Address: \2%s\2",
	      me.name, RPL_SETTINGS, sptr->name, KLINE_ADDRESS);

  sendto_one (sptr, ":%s %d %s :Usermode x locked: \2%i\2",
	      me.name, RPL_SETTINGS, sptr->name, MODE_x);

  sendto_one (sptr, ":%s %d %s :Network Help Channel: \2%s\2",
	      me.name, RPL_SETTINGS, sptr->name, helpchan);

  sendto_one (sptr, ":%s %d %s : ", me.name, RPL_SETTINGS, sptr->name);


  sendto_one (sptr,
	      ":%s %d %s :\2~~~~~~~~~~~~~~~~~~~~~~~~~~\2",
	      me.name, RPL_SETTINGS, sptr->name);


  sendto_one (sptr, rpl_str (RPL_ENDOFSETTINGS), me.name, sptr->name);
}

/* Report Options */
void
report_options (aClient * sptr)
{
  sendto_one (sptr,
	      ":%s %d %s :\2~~~~~~~~ Network Options ~~~~~~~~\2",
	      me.name, RPL_SETTINGS, sptr->name);

  sendto_one (sptr, ":%s %d %s : ", me.name, RPL_SETTINGS, sptr->name);

  sendto_one (sptr, ":%s %d %s :Clone Protection: \2%i\2",
	      me.name, RPL_SETTINGS, sptr->name, net_clone_protection);

  sendto_one (sptr, ":%s %d %s :Allowed Connections Per Host: \2%d\2",
	      me.name, RPL_SETTINGS, sptr->name, net_clone_limit);

  sendto_one (sptr, ":%s %d %s :Oper Bypass Connection Limit: \2%i\2",
	      me.name, RPL_SETTINGS, sptr->name, net_clone_limit_bypass);

  sendto_one (sptr, ":%s %d %s :Hide U Lines: \2%i\2",
	      me.name, RPL_SETTINGS, sptr->name, HIDE_ULINES);

  sendto_one (sptr, ":%s %d %s :Show Global Connect Notices: \2%i\2",
	      me.name, RPL_SETTINGS, sptr->name, GLOBAL_CONNECTS);

  sendto_one (sptr, ":%s %d %s :Notify On Proxy Scan Excempts (e): \2%i\2",
	      me.name, RPL_SETTINGS, sptr->name, network_proxyscan_notice);

  sendto_one (sptr, ":%s %d %s :Auto Set +p (Protected Oper): \2%i\2",
	      me.name, RPL_SETTINGS, sptr->name, network_auto_protect);

  sendto_one (sptr, ":%s %d %s :Users Auto Join Channel(s): \2%s\2",
	      me.name, RPL_SETTINGS, sptr->name, net_users_auto_join);

  sendto_one (sptr, ":%s %d %s :Opers Auto Join Channel(s): \2%s\2",
	      me.name, RPL_SETTINGS, sptr->name, net_opers_auto_join);

  sendto_one (sptr, ":%s %d %s : ", me.name, RPL_SETTINGS, sptr->name);


  sendto_one (sptr,
	      ":%s %d %s :\2~~~~~~~~~~~~~~~~~~~~~~~~~~\2",
	      me.name, RPL_SETTINGS, sptr->name);


  sendto_one (sptr, rpl_str (RPL_ENDOFSETTINGS), me.name, sptr->name);
}


/* Report Hostmasks */
void
report_hostmasks (aClient * sptr)
{
  sendto_one (sptr,
	      ":%s %d %s :\2~~~~~~~~ Network Hostmasks ~~~~~~~~\2",
	      me.name, RPL_SETTINGS, sptr->name);

  sendto_one (sptr, ":%s %d %s : ", me.name, RPL_SETTINGS, sptr->name);

  sendto_one (sptr, ":%s %d %s :Local Operator Hostname: \2%s\2",
	      me.name, RPL_SETTINGS, sptr->name, locop_host);

  sendto_one (sptr, ":%s %d %s :Global Operator Hostname: \2%s\2",
	      me.name, RPL_SETTINGS, sptr->name, oper_host);

  sendto_one (sptr, ":%s %d %s :Administrator Hostname: \2%s\2",
	      me.name, RPL_SETTINGS, sptr->name, admin_host);

  sendto_one (sptr, ":%s %d %s :Server Administrator Hostname: \2%s\2",
	      me.name, RPL_SETTINGS, sptr->name, servadmin_host);

  sendto_one (sptr, ":%s %d %s :Services Oper Hostname: \2%s\2",
	      me.name, RPL_SETTINGS, sptr->name, servicesoper_host);

  sendto_one (sptr, ":%s %d %s :Services Administrator Hostname: \2%s\2",
	      me.name, RPL_SETTINGS, sptr->name, servicesadmin_host);

  sendto_one (sptr, ":%s %d %s :Services Root Hostname: \2%s\2",
	      me.name, RPL_SETTINGS, sptr->name, servicesroot_host);

  sendto_one (sptr, ":%s %d %s :Technical Administrator Hostname: \2%s\2",
	      me.name, RPL_SETTINGS, sptr->name, techadmin_host);

  sendto_one (sptr, ":%s %d %s :Network Administrator Hostname: \2%s\2",
	      me.name, RPL_SETTINGS, sptr->name, netadmin_host);

  sendto_one (sptr, ":%s %d %s :IRC Administrator Hostname: \2%s\2",
	      me.name, RPL_SETTINGS, sptr->name, ircadmin_host);

  sendto_one (sptr, ":%s %d %s :Server Bot Hostname: \2%s\2",
	      me.name, RPL_SETTINGS, sptr->name, sbot_host);

  sendto_one (sptr, ":%s %d %s :Network Monitor Bot Hostname: \2%s\2",
	      me.name, RPL_SETTINGS, sptr->name, netmon_host);

  sendto_one (sptr, ":%s %d %s : ", me.name, RPL_SETTINGS, sptr->name);


  sendto_one (sptr,
	      ":%s %d %s :\2~~~~~~~~~~~~~~~~~~~~~~~~~~\2",
	      me.name, RPL_SETTINGS, sptr->name);


  sendto_one (sptr, rpl_str (RPL_ENDOFSETTINGS), me.name, sptr->name);
}

/* Report services */
void
report_services (aClient * sptr)
{
  sendto_one (sptr,
	      ":%s %d %s :\2~~~~~~~~ Services Settings ~~~~~~~~\2",
	      me.name, RPL_SETTINGS, sptr->name);

  sendto_one (sptr, ":%s %d %s : ", me.name, RPL_SETTINGS, sptr->name);

  sendto_one (sptr, ":%s %d %s :Services Server: \2%s\2",
	      me.name, RPL_SETTINGS, sptr->name, services_name);

  sendto_one (sptr, ":%s %d %s :WatchDog Server: \2%s\2",
	      me.name, RPL_SETTINGS, sptr->name, watchdog_name);

  sendto_one (sptr, ":%s %d %s :StatServ Server: \2%s\2",
	      me.name, RPL_SETTINGS, sptr->name, statserv_name);

  sendto_one (sptr, ":%s %d %s : ", me.name, RPL_SETTINGS, sptr->name);


  sendto_one (sptr,
	      ":%s %d %s :\2~~~~~~~~~~~~~~~~~~~~~~~~~~\2",
	      me.name, RPL_SETTINGS, sptr->name);


  sendto_one (sptr, rpl_str (RPL_ENDOFSETTINGS), me.name, sptr->name);
}



/* Report Misc */
void
report_misc (aClient * sptr)
{
  sendto_one (sptr,
	      ":%s %d %s :\2~~~~~~~~ Misc Network Settings ~~~~~~~~\2",
	      me.name, RPL_SETTINGS, sptr->name);

  sendto_one (sptr, ":%s %d %s : ", me.name, RPL_SETTINGS, sptr->name);

  sendto_one (sptr, ":%s %d %s :Network Enforcer: \2%s (%s@%s)\2",
	      me.name, RPL_SETTINGS, sptr->name, net_enforcer,
	      net_enforcer_userid, net_enforcer_host);

  sendto_one (sptr, ":%s %d %s :Network Enforcer Realname: \2%s\2",
	      me.name, RPL_SETTINGS, sptr->name, net_enforcer_realname);

  sendto_one (sptr, ":%s %d %s :Network Enforcer Server: \2%s\2 ",
	      me.name, RPL_SETTINGS, sptr->name, net_enforcer_server);

  sendto_one (sptr, ":%s %d %s :Network Enforcer Server Realname: \2%s\2",
	      me.name, RPL_SETTINGS, sptr->name, net_enforcer_server_name);

  sendto_one (sptr, ":%s %d %s : ", me.name, RPL_SETTINGS, sptr->name);


  sendto_one (sptr,
	      ":%s %d %s :\2~~~~~~~~~~~~~~~~~~~~~~~~~~\2",
	      me.name, RPL_SETTINGS, sptr->name);


  sendto_one (sptr, rpl_str (RPL_ENDOFSETTINGS), me.name, sptr->name);
}

void
check_dynconf (int type)
{
/*
** Dynconf Error checking backported from Uircd3
** We make sure that there are no stupid or missing values.
*/

/*
** Network Template Settings
*/

/*
** IRCNETWORK
*/

  if (ircnetwork == NULL)
    {
      AllocCpy (ircnetwork, "SomeIRC");

      if (type == 0)
	fprintf (stderr,
		 " Dynconf ERROR: Set ircnetwork in %s does not exist. Setting it to default.\n",
		 INCLUDE);
      else
	sendto_realops
	  ("Dynconf ERROR: Set ircnetwork in %s does not exist. Setting it to default.",
	   INCLUDE);
    }

/*
** IRCNETWORK_NAME
*/

  if (ircnetwork_name == NULL)
    {
      AllocCpy (ircnetwork_name, "The SomeIRC Network");

      if (type == 0)
	fprintf (stderr,
		 " Dynconf ERROR: Set ircnetwork_name in %s does not exist. Setting it to default.\n",
		 INCLUDE);
      else
	sendto_realops
	  ("Dynconf ERROR: Set ircnetwork_name in %s does not exist. Setting it to default.",
	   INCLUDE);
    }

/*
** DEFSERV
*/

  if (defserv == NULL)
    {
      AllocCpy (defserv, "irc.someirc.net");

      if (type == 0)
	fprintf (stderr,
		 " Dynconf ERROR: Set defserv in %s does not exist. Setting it to default.\n",
		 INCLUDE);
      else
	sendto_realops
	  ("Dynconf ERROR: Set defserv in %s does not exist. Setting it to default.",
	   INCLUDE);
    }

/*
** WEBPAGE
*/

  if (webpage == NULL)
    {
      AllocCpy (webpage, "http://www.someirc.net");

      if (type == 0)
	fprintf (stderr,
		 " Dynconf ERROR: Set webpage in %s does not exist. Setting it to default.\n",
		 INCLUDE);
      else
	sendto_realops
	  ("Dynconf ERROR: Set webpage in %s does not exist. Setting it to default.",
	   INCLUDE);
    }

/*
** WEBPAGE_RULES
*/

  if (webpage_rules == NULL)
    {
      AllocCpy (webpage_rules, "http://www.someirc.net/rules/");

      if (type == 0)
	fprintf (stderr,
		 " Dynconf ERROR: Set webpagerules in %s does not exist. Setting it to default.\n",
		 INCLUDE);
      else
	sendto_realops
	  ("Dynconf ERROR: Set webpagerules in %s does not exist. Setting it to default.",
	   INCLUDE);
    }

/*
** KLINE_ADDRESS
*/

  if (KLINE_ADDRESS == NULL)
    {
      AllocCpy (KLINE_ADDRESS, "kline@someirc.net");

      if (type == 0)
	fprintf (stderr,
		 " Dynconf ERROR: Set ircnetwork in %s does not exist. Setting it to default.\n",
		 INCLUDE);
      else
	sendto_realops
	  ("Dynconf ERROR: Set ircnetwork in %s does not exist. Setting it to default.",
	   INCLUDE);
    }

/*
** SERVICES_NAME
*/

  if (services_name == NULL)
    {
      AllocCpy (services_name, "services.someirc.net");

      if (type == 0)
	fprintf (stderr,
		 " Dynconf ERROR: Set services_name in %s does not exist. Setting it to default.\n",
		 INCLUDE);
      else
	sendto_realops
	  ("Dynconf ERROR: Set services_name in %s does not exist. Setting it to default.",
	   INCLUDE);
    }

/*
** WATCHDOG_NAME
*/

  if (watchdog_name == NULL)
    {
      AllocCpy (watchdog_name, "watchdog.someirc.net");

      if (type == 0)
	fprintf (stderr,
		 " Dynconf ERROR: Set watchdog_name in %s does not exist. Setting it to default.\n",
		 INCLUDE);
      else
	sendto_realops
	  ("Dynconf ERROR: Set watchdog_name in %s does not exist. Setting it to default.",
	   INCLUDE);
    }

/*
** STATSERV_NAME
*/

  if (statserv_name == NULL)
    {
      AllocCpy (statserv_name, "statistics.someirc.net");

      if (type == 0)
	fprintf (stderr,
		 " Dynconf ERROR: Set statserv_name in %s does not exist. Setting it to default.\n",
		 INCLUDE);
      else
	sendto_realops
	  ("Dynconf ERROR: Set statserv_name in %s does not exist. Setting it to default.",
	   INCLUDE);
    }

/*
** LOCOP_HOST
*/

  if (locop_host == NULL)
    {
      AllocCpy (locop_host, "LocalOper.someirc.net");

      if (type == 0)
	fprintf (stderr,
		 " Dynconf ERROR: Set locop_host in %s does not exist. Setting it to default.\n",
		 INCLUDE);
      else
	sendto_realops
	  ("Dynconf ERROR: Set locop_host in %s does not exist. Setting it to default.",
	   INCLUDE);
    }

/*
** OPER_HOST
*/

  if (oper_host == NULL)
    {
      AllocCpy (oper_host, "Oper.someirc.net");

      if (type == 0)
	fprintf (stderr,
		 " Dynconf ERROR: Set oper_host in %s does not exist. Setting it to default.\n",
		 INCLUDE);
      else
	sendto_realops
	  ("Dynconf ERROR: Set oper_host in %s does not exist. Setting it to default.",
	   INCLUDE);
    }

/*
** ADMIN_HOST
*/

  if (admin_host == NULL)
    {
      AllocCpy (admin_host, "Admin.someirc.net");

      if (type == 0)
	fprintf (stderr,
		 " Dynconf ERROR: Set admin_host in %s does not exist. Setting it to default.\n",
		 INCLUDE);
      else
	sendto_realops
	  ("Dynconf ERROR: Set admin_host in %s does not exist. Setting it to default.",
	   INCLUDE);
    }

/*
** SERVADMIN_HOST
*/

  if (servadmin_host == NULL)
    {
      AllocCpy (servadmin_host, "ServerAdmin.someirc.net");

      if (type == 0)
	fprintf (stderr,
		 " Dynconf ERROR: Set servadmin_host in %s does not exist. Setting it to default.\n",
		 INCLUDE);
      else
	sendto_realops
	  ("Dynconf ERROR: Set servadmin_host in %s does not exist. Setting it to default.",
	   INCLUDE);
    }

/*
** SERVICESOPER_HOST
*/

  if (servicesoper_host == NULL)
    {
      AllocCpy (servicesoper_host, "ServicesOper.someirc.net");

      if (type == 0)
	fprintf (stderr,
		 " Dynconf ERROR: Set servicesoper_host in %s does not exist. Setting it to default.\n",
		 INCLUDE);
      else
	sendto_realops
	  ("Dynconf ERROR: Set servicesoper_host in %s does not exist. Setting it to default.",
	   INCLUDE);
    }

/*
** SERVICESADMIN_HOST
*/

  if (servicesadmin_host == NULL)
    {
      AllocCpy (servicesadmin_host, "ServicesAdmin.someirc.net");

      if (type == 0)
	fprintf (stderr,
		 " Dynconf ERROR: Set servicesadmin_host in %s does not exist. Setting it to default.\n",
		 INCLUDE);
      else
	sendto_realops
	  ("Dynconf ERROR: Set servicesadmin_host in %s does not exist. Setting it to default.",
	   INCLUDE);
    }

/*
** SERVICESROOT_HOST
*/

  if (servicesroot_host == NULL)
    {
      AllocCpy (servicesroot_host, "ServicesRoot.someirc.net");

      if (type == 0)
	fprintf (stderr,
		 " Dynconf ERROR: Set servicesroot_host in %s does not exist. Setting it to default.\n",
		 INCLUDE);
      else
	sendto_realops
	  ("Dynconf ERROR: Set servicesroot_host in %s does not exist. Setting it to default.",
	   INCLUDE);
    }

/*
** TECHADMIN_HOST
*/

  if (techadmin_host == NULL)
    {
      AllocCpy (techadmin_host, "TechAdmin.someirc.net");

      if (type == 0)
	fprintf (stderr,
		 " Dynconf ERROR: Set techadmin_host in %s does not exist. Setting it to default.\n",
		 INCLUDE);
      else
	sendto_realops
	  ("Dynconf ERROR: Set techadmin_host in %s does not exist. Setting it to default.",
	   INCLUDE);
    }

/*
** NETADMIN_HOST
*/

  if (netadmin_host == NULL)
    {
      AllocCpy (netadmin_host, "NetAdmin.someirc.net");

      if (type == 0)
	fprintf (stderr,
		 " Dynconf ERROR: Set netadmin_host in %s does not exist. Setting it to default.\n",
		 INCLUDE);
      else
	sendto_realops
	  ("Dynconf ERROR: Set netadmin_host in %s does not exist. Setting it to default.",
	   INCLUDE);
    }

/*
** IRCADMIN_HOST
*/

  if (ircadmin_host == NULL)
    {
      AllocCpy (ircadmin_host, "IRCAdmin.someirc.net");

      if (type == 0)
	fprintf (stderr,
		 " Dynconf ERROR: Set ircadmin_host in %s does not exist. Setting it to default.\n",
		 INCLUDE);
      else
	sendto_realops
	  ("Dynconf ERROR: Set ircadmin_host in %s does not exist. Setting it to default.",
	   INCLUDE);
    }

/*
** SBOT_HOST
*/

  if (sbot_host == NULL)
    {
      AllocCpy (sbot_host, "ServerBot.someirc.net");

      if (type == 0)
	fprintf (stderr,
		 " Dynconf ERROR: Set sbot_host in %s does not exist. Setting it to default.\n",
		 INCLUDE);
      else
	sendto_realops
	  ("Dynconf ERROR: Set sbot_host in %s does not exist. Setting it to default.",
	   INCLUDE);
    }

/*
** NETMON_HOST
*/

  if (netmon_host == NULL)
    {
      AllocCpy (netmon_host, "NetworkMonitor.someirc.net");

      if (type == 0)
	fprintf (stderr,
		 " Dynconf ERROR: Set netmon_host in %s does not exist. Setting it to default.\n",
		 INCLUDE);
      else
	sendto_realops
	  ("Dynconf ERROR: Set netmon_host in %s does not exist. Setting it to default.",
	   INCLUDE);
    }

/*
** HELPCHAN
*/

  if (helpchan == NULL)
    {
      AllocCpy (helpchan, "#help");

      if (type == 0)
	fprintf (stderr,
		 " Dynconf ERROR: Set helpchan in %s does not exist. Setting it to default.\n",
		 INCLUDE);
      else
	sendto_realops
	  ("Dynconf ERROR: Set helpchan in %s does not exist. Setting it to default.",
	   INCLUDE);
    }

/*
** NET_ENFORCER
*/

  if (net_enforcer == NULL)
    {
      AllocCpy (net_enforcer, "Enforcer");

      if (type == 0)
	fprintf (stderr,
		 " Dynconf ERROR: Set net_enforcer in %s does not exist. Setting it to default.\n",
		 INCLUDE);
      else
	sendto_realops
	  ("Dynconf ERROR: Set net_enforcer in %s does not exist. Setting it to default.",
	   INCLUDE);
    }

/*
** NET_ENFORCER_USERID
*/

  if (net_enforcer_userid == NULL)
    {
      AllocCpy (net_enforcer_userid, "enforcer");

      if (type == 0)
	fprintf (stderr,
		 " Dynconf ERROR: Set net_enforcer_userid in %s does not exist. Setting it to default.\n",
		 INCLUDE);
      else
	sendto_realops
	  ("Dynconf ERROR: Set net_enforcer_userid in %s does not exist. Setting it to default.",
	   INCLUDE);
    }

/*
** NET_ENFORCER_HOST
*/

  if (net_enforcer_host == NULL)
    {
      AllocCpy (net_enforcer_host, "Enforcer.someirc.net");

      if (type == 0)
	fprintf (stderr,
		 " Dynconf ERROR: Set net_enforcer_host in %s does not exist. Setting it to default.\n",
		 INCLUDE);
      else
	sendto_realops
	  ("Dynconf ERROR: Set net_enforcer_host in %s does not exist. Setting it to default.",
	   INCLUDE);
    }

/*
** NET_ENFORCER_REALNAME
*/

  if (net_enforcer_realname == NULL)
    {
      AllocCpy (net_enforcer_realname, "Enforcer");

      if (type == 0)
	fprintf (stderr,
		 " Dynconf ERROR: Set net_enforcer_realname in %s does not exist. Setting it to default.\n",
		 INCLUDE);
      else
	sendto_realops
	  ("Dynconf ERROR: Set net_enforcer_realname in %s does not exist. Setting it to default.",
	   INCLUDE);
    }

/*
** NET_ENFORCER_SERVER
*/

  if (net_enforcer_server == NULL)
    {
      AllocCpy (net_enforcer_server, "Enforcer.someirc.net");

      if (type == 0)
	fprintf (stderr,
		 " Dynconf ERROR: Set net_enforcer_server in %s does not exist. Setting it to default.\n",
		 INCLUDE);
      else
	sendto_realops
	  ("Dynconf ERROR: Set net_enforcer_server in %s does not exist. Setting it to default.",
	   INCLUDE);
    }

/*
** NET_ENFORCER_SERVER_NAME
*/

  if (net_enforcer_server_name == NULL)
    {
      AllocCpy (net_enforcer_server_name, "The SomeIRC Enforcer");

      if (type == 0)
	fprintf (stderr,
		 " Dynconf ERROR: Set net_enforcer_server_name in %s does not exist. Setting it to default.\n",
		 INCLUDE);
      else
	sendto_realops
	  ("Dynconf ERROR: Set net_enforcer_server_name in %s does not exist. Setting it to default.",
	   INCLUDE);
    }


/*
** ircd.ini Settings
*/

/*
** CLIENT_FLOOD
*/

  if (!CLIENT_FLOOD)
    {
      CLIENT_FLOOD = atol ("4000");

      if (type == 0)
	fprintf (stderr,
		 " Dynconf ERROR: Set client_flood in ircd.ini does not exist. Setting it to default.\n",
		 INCLUDE);
      else
	sendto_realops
	  ("Dynconf ERROR: Set client_flood in ircd.ini does not exist. Setting it to default.",
	   INCLUDE);
    }

/*
** GEO_LOCATION
*/

  if (geo_location == NULL)
    {
      AllocCpy (geo_location, "Somewhere between you and the sun");

      if (type == 0)
	fprintf (stderr,
		 " Dynconf ERROR: Set geo_location in ircd.ini does not exist. Setting it to default.\n",
		 INCLUDE);
      else
	sendto_realops
	  ("Dynconf ERROR: Set geo_location in ircd.ini does not exist. Setting it to default.",
	   INCLUDE);
    }


  if (type == 0)
    (void) fprintf (stderr, " Dynconf settings verified.\n");
  else
    sendto_realops ("Dynconf settings verified.");

}
