/*
 *  UltimateIRCd - an Internet Relay Chat Daemon, src/s_socks.c
 *
 *  Copyright (C) 2002 by the past and present ircd coders, and others.
 *  Refer to the documentation within doc/authors/ for full credits and copyrights.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *  USA
 *
 *  $Id: s_socks.c,v 1.5 2002/11/18 00:31:31 shadowmaster Exp $
 *
 */

/* All these includes are copied from auth.c because we are
   just doing something very similar */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "res.h"
#include "numeric.h"
#include "version.h"
#ifndef _WIN32
#include <sys/socket.h>
#include <sys/file.h>
#include <sys/ioctl.h>
#ifdef	UNIXPORT
# include <sys/un.h>
#endif
#if defined(__hpux)
# include "inet.h"
#endif
#else
#include <io.h>
#endif
#include <fcntl.h>
#include "sock.h"		/* If FD_ZERO isn't define up to this point,  */
			/* define it (BSD4.2 needs this) */
#include "h.h"

#ifdef SOCKSPORT

static unsigned char socksid[12];

#define HICHAR(s)	(((unsigned short) s) >> 8)
#define LOCHAR(s)	(((unsigned short) s) & 0xFF)


/*
 * start_socks
 *
 * attempts to connect to a socks server on port 1080 of the host.
 * if the connect fails, the user passes the socks check
 * otherwise, the connection is checked to see if it is a "secure"
 * socks4+ server
 */
void
start_socks (cptr)
     aClient *cptr;
{
  struct sockaddr_in sock;
  int addrlen = sizeof (struct sockaddr_in);

  struct sockaddr_in sin;
  int sinlen = sizeof (struct sockaddr_in);

  if (find_socksexcept ((char *) inetntoa ((char *) &cptr->ip)))
    {
#ifdef SHOWCONNECTINFO
      write (cptr->fd, REPORT_SOCKSEXCEPT, R_socksexcept);
#endif
      if (network_proxyscan_notice == 1)
	{
	  sendto_serv_butone (&me,
			      ":%s NETG :\2[Excempt Socks]\2 Client bypassing Proxy Scan connecting from %s",
			      me.name, get_client_host (cptr));
	  sendto_netglobal
	    ("from %s : \2[Excempt Socks]\2 Client bypassing Proxy Scan connecting from %s",
	     me.name, get_client_host (cptr));
	}
      return;
    }

  Debug ((DEBUG_NOTICE, "start_socks(%x) fd %d status %d",
	  cptr, cptr->fd, cptr->status));
  if ((cptr->socksfd = socket (AF_INET, SOCK_STREAM, 0)) == -1)
    {
#ifdef	USE_SYSLOG
      syslog (LOG_ERR, "Unable to create socks socket for %s:%m",
	      get_client_name (cptr, TRUE));
#endif
      Debug ((DEBUG_ERROR, "Unable to create auth socket for %s:%s",
	      get_client_name (cptr, TRUE), strerror (get_sockerr (cptr))));
      return;
    }
#ifndef _WIN32
  if (cptr->socksfd >= (MAXCONNECTIONS - 3))
    {
      sendto_ops ("Can't allocate fd for socks on %s",
		  get_client_name (cptr, TRUE));
      (void) close (cptr->socksfd);
      return;
    }
#endif

#ifdef SHOWCONNECTINFO
  write (cptr->fd, REPORT_DO_SOCKS, R_do_socks);
#endif

  set_non_blocking (cptr->socksfd, cptr);

  getsockname (cptr->fd, (struct sockaddr *) &sock, &addrlen);
  sock.sin_port = 0;
  sock.sin_family = AF_INET;	/* redundant? */
  (void) bind (cptr->socksfd, (struct sockaddr *) &sock, sizeof (sock));

  bcopy ((char *) &cptr->ip, (char *) &sock.sin_addr,
	 sizeof (struct in_addr));

  sock.sin_port = htons (1080);
  sock.sin_family = AF_INET;

  if (connect (cptr->socksfd, (struct sockaddr *) &sock,
#ifndef _WIN32
	       sizeof (sock)) == -1 && errno != EINPROGRESS)
#else
	       sizeof (sock)) == -1 && (WSAGetLastError () !=
					WSAEINPROGRESS
					&& WSAGetLastError () !=
					WSAEWOULDBLOCK))
#endif
  {
#ifndef _WIN32
    (void) close (cptr->socksfd);
#else
    (void) closesocket (cptr->socksfd);
#endif
    cptr->socksfd = -1;
#ifdef SHOWCONNECTINFO
    write (cptr->fd, REPORT_NO_SOCKS, R_no_socks);
#endif
    return;
  }
  cptr->flags |= (FLAGS_WRSOCKS | FLAGS_SOCKS);
  if (cptr->socksfd > highest_fd)
    highest_fd = cptr->socksfd;
  return;
}

/*
 * send_socksquery
 *
 * send the socks server a query to see if it's open.
 */
void
send_socksquery (cptr)
     aClient *cptr;
{
  struct sockaddr_in sin;
  int sinlen = sizeof (struct sockaddr_in);
  unsigned char socksbuf[12];
  unsigned long theip;

  bcopy ((char *) &socksid, (char *) &socksbuf, 9);

  getsockname (cptr->fd, (struct sockaddr *) &sin, &sinlen);

  theip = htonl (sin.sin_addr.s_addr);

  socksbuf[4] = (theip >> 24);
  socksbuf[5] = (theip >> 16) & 0xFF;
  socksbuf[6] = (theip >> 8) & 0xFF;
  socksbuf[7] = theip & 0xFF;

  if (send (cptr->socksfd, socksbuf, 9, 0) != 9)
    {
#ifndef _WIN32
      close (cptr->socksfd);
#else
      closesocket (cptr->socksfd);
#endif
      if (cptr->socksfd == highest_fd)
	while (!local[highest_fd])
	  highest_fd--;
      cptr->socksfd = -1;
      cptr->flags &= ~FLAGS_SOCKS;
#ifdef SHOWCONNECTINFO
      write (cptr->fd, REPORT_NO_SOCKS, R_no_socks);
#endif
    }
  cptr->flags &= ~FLAGS_WRSOCKS;
  return;
}

/*
 * read_socks
 *
 * process the socks reply.
 */

void
read_socks (cptr)
     aClient *cptr;
{
  unsigned char socksbuf[12];
  int len;

  len = recv (cptr->socksfd, socksbuf, 9, 0);

#ifndef _WIN32
  (void) close (cptr->socksfd);
#else
  (void) closesocket (cptr->socksfd);
#endif
  if (cptr->socksfd == highest_fd)
    while (!local[highest_fd])
      highest_fd--;
  cptr->socksfd = -1;
  ClearSocks (cptr);

  if (len < 4)
    {
#ifdef SHOWCONNECTINFO
      write (cptr->fd, REPORT_NO_SOCKS, R_no_socks);
#endif
      return;
    }

  if (socksbuf[1] == 90)
    {
      cptr->flags |= FLAGS_GOTSOCKS;
      return;
    }

#ifdef SHOWCONNECTINFO
  write (cptr->fd, REPORT_GOOD_SOCKS, R_good_socks);
#endif
  return;
}

#endif /* SOCKSPORT */
